#!/usr/bin/perl -w

# browse.pl

# as item.pl works on single lists, this is the primary lists for working with
# several lists (groupings of lists) at once.

# it is capable of the followintg actions:
#    browsetype ->  browsing accepts a list of filters and and a browseby type
#                   which can not be one of the filters and will show that
#                   level
#    listing    ->  listing accepts a list of filters and returns all lists
#    search_keyword  -> a very simple search method for searching the database

use strict;
use CGI qw/:standard/;                   # for cookies and reading params
use Calf::Auth qw( :base );                # Site authentication module
use Calf::Config qw( :base );              # basic site config module
use Calf::Utils qw( :base :template :db ); # basic utilities

my $dbh = Calf::Utils::open_db;
my $sth;
my $q = new CGI;
my $form_info = $q->Vars; # put the form info into a hash
my ( $i, $ii );

# is the person logged in? $session_info is a refernce to a hash
# containing the username and session_id of the person logged in

my $session_info = &valid_id ($q, $dbh);
my $action = $form_info->{action};
my $national_only = $form_info->{nationalonly};

$| = 1;

# chose which page to render then go and render that page

if ( $action eq 'browsetype' )
  { &browsetype; }
elsif ( $action eq 'listing' )
  { &listing; }
elsif ( $action eq 'search' )
  { &search_keyword }
else
  { &error( 'none', 'invalidaction' ); }

$dbh->disconnect;  # disconnect from the database and end the program

## SUBROUTINES
############################################
## Actions Defined for this Script
# possible actions: browsetype, listing

# browsetype was represented by in the old code by const.php3 and
# issues.php3 but is much more powerful in this version, it takes
# input in the following form:

# action=browsetype (required)
# type1=category1 (optional) (one or more category codes)
# type2=category2  ""              ""                   ""
# browsetype=type3 (required)

# at this point there are only two types: issues, and constit but
# there might be more in golden or in another version right now the
# only possibilities are issues=A1&browsetype=Constit or even just
# browsetype=Constit (where you will browse all lists by constit

sub browsetype {
  my $descriptions = &build_descriptions( $dbh );
  my ( @categories );
  my ( $select_statement, $conditions_fragment ) = ();

  my $browse_type = $form_info->{browseby};

  my $editor = 1 if ( defined( $session_info ) );

  my $filters = &get_filters;
  undef $filters if ( scalar( keys %$filters ) < 1 );

  $conditions_fragment = &build_conditions_fragment( $filters );

  # grab the total number of lists matching the criteria
  my $total_count;
  $select_statement = q/ SELECT COUNT(*) AS CNT FROM calf_data /;
  $select_statement .= qq/ WHERE recommended <= $rec_thresh AND approved = 1 /;
  $select_statement .= q/ AND / if ( defined( $filters ) );
  $select_statement .= $conditions_fragment;
  $total_count = ( $dbh->selectrow_array( $select_statement ) )[0]
    || die "can't call selectrow_array on $select_statement: $!";

  # grab the constituencies that they fall into and the number of
  # groups in each constits
  $select_statement = qq/ SELECT
                          $browse_type.description AS description,
                          $browse_type.code AS code,
                          COUNT($browse_type.description) AS cnt
                          FROM calf_data, $browse_type
                          WHERE
                          recommended <= $rec_thresh
                          AND approved = 1
                        /;
  if ( defined( $filters ) ) {
    $select_statement .= q/ AND /;
    $select_statement .= $conditions_fragment;
  }
  $select_statement .= qq/ AND calf_data.$browse_type LIKE
                           CONCAT('%' , $browse_type.code , '%' ) /;
  $select_statement .= qq/ GROUP BY $browse_type.description /;

  $sth = $dbh->prepare( $select_statement )
    || die "can't prepare database handle; $!";
  $sth->execute
    || die "can't execute database statement $select_statement: $!";
  while ( my $hash_ref = $sth->fetchrow_hashref( 'NAME_lc' ) ) {
    push @categories, { %$hash_ref };
  }

  # go into listing mode if the lists fall into less than two categories
  ( &listing( '' ) && return ) if ( scalar( @categories ) <= 2 );
#				    && $total_count < 22 );

  # copy data into the outgoing hash
  my $vars = { session_info => $session_info,
	       editor => $editor,
	       national_only => $national_only,
	       type_defs => \%type_defs,
	       descriptions => $descriptions,
	       filters => $filters,
	       categories => \@categories,
	       total_count => $total_count,
	       browseby => $browse_type,
	       page => "browsetype"
	     };

  # process the cookie and print the page
  &process_cookie( $session_info->{session_id} );
  &process_template('browse/browsetype.html', $vars );
}


## listing

# the idea behind listing is a bit mroe simple than browse.
# listing takes simple a list of filters specified as type=cateogry in the url
# and then return a list of all the lists in teh database that fit these 
# criteria

sub listing {
  my $editor = 1 if ( defined( $session_info ) );
  my $descriptions = &build_descriptions( $dbh );
  my ( @lists );
  my ( $select_statement, $conditions_fragment ) = ();
  my ( $type );

  my $filters = &get_filters;
  $conditions_fragment = &build_conditions_fragment( $filters );

  # grab the total number of lists matching the criteria
  my $total_count;
  $select_statement = qq/ SELECT COUNT(*) AS CNT FROM calf_data
                          WHERE recommended <= $rec_thresh
                          AND approved = 1 /;
  $select_statement .= q/ AND / if ( defined( $filters ) );
  $select_statement .= $conditions_fragment;
  $total_count = ( $dbh->selectrow_array( $select_statement ) )[0]
    || die "failed to execute database statement $select_statement: $!";

  # grab the list information out of the db
  $select_statement = q/ SELECT /;

  for ( $i = 0; $i <= $#all_fields; $i++ ) {
    my $field = $all_fields[$i];
    $select_statement .= qq/ $field AS $field /;
    $select_statement .= "," unless $i == $#all_fields;
  }
  $select_statement .= q/ FROM calf_data /;

  $select_statement .= qq/ WHERE recommended <= $rec_thresh AND approved = 1/;
  $select_statement .= q/ AND / if ( defined( $filters ) );
  $select_statement .= $conditions_fragment;

  $select_statement .= q/ ORDER BY name /;

  $sth = $dbh->prepare( $select_statement )
    || &db_error( "prepare", $!, $select_statement );
  $sth->execute
    || &db_error( "execute", $!, $select_statement );
  while ( my $hash_ref = $sth->fetchrow_hashref('NAME_lc') ) {
    foreach $type (@types) {
      $hash_ref->{"$type"} = [ &parse_codes( "$hash_ref->{$type}" ) ];
    }
    push @lists, { %$hash_ref };
  }

  # copy data into the outgoing hash
  my $vars = { session_info => $session_info,
	       editor => $editor,
	       national_only => $national_only,
	       descriptions => $descriptions,
	       filters => $filters,
	       total_count => $total_count,
	       lists => \@lists,
	       page => "listing"
	     };
  # process the cookie and print the page
  &process_cookie( $session_info->{session_id} );
  &process_template('browse/listing.html', $vars );

}

# search the lists by keyword
sub search_keyword {
  my $editor;
  my $descriptions = &build_descriptions( $dbh );
  my ( $select_statement, $conditions_fragment ) = ();
  my $hit_count = 0;
  my ( $filters, $type, @lists );

  $editor = 1 if ( defined( $session_info ) );

  if (chomp( $form_info->{"searchtext"} ) eq "" ) {
    my $vars = { session_info => $session_info,
		 editor => $editor,
		 filters => $filters,
		 searchtext => $form_info->{searchtext},
		 page => "search",
		 failure => "notext"
	       };

    # process the cookie and print the page
    &process_cookie( $session_info->{session_id} );
    &process_template('search_results.html', $vars );
    return;
  }

  if ( defined $form_info->{restrict}
       && $form_info->{restrict} == 1 ) {
    $filters = &get_filters;
    $conditions_fragment = &build_conditions_fragment( $filters );
  }

  # grab the total number of lists matching the criteria
  my $total_count;
  $select_statement = q/ SELECT COUNT(*) AS CNT FROM calf_data /;
  $select_statement .= qq/ WHERE recommended <= $rec_thresh AND approved = 1 /;
  if ( defined( $filters ) )
    { $select_statement .= q/ AND /;
      $select_statement .= $conditions_fragment; }

  $total_count = ( $dbh->selectrow_array( $select_statement ) )[0]
    || &db_error( "prepare/execute", $!, $select_statement );

  $select_statement = qq/ SELECT /;
  for ( $i = 0; $i <= $#all_fields; $i++ ) {
    my $field = $all_fields[$i];
    $select_statement .= qq/ $field AS $field /;
    $select_statement .= "," unless $i == $#all_fields;
  }
  $select_statement .= qq/ FROM calf_data
                           WHERE
                           recommended <= $rec_thresh
                           AND approved = 1
                         /;

  if ( $form_info->{restrict} == 1 && defined $filters ) {
    $select_statement .= q/ AND /;
    $select_statement .= $conditions_fragment;
  }
  $select_statement .= q/ AND /;
  $select_statement .= qq/ ( keywords LIKE "%$form_info->{searchtext}%"
                           OR name LIKE "%$form_info->{searchtext}%" )
                           ORDER BY name
                         /;
  $sth = $dbh->prepare( $select_statement )
    || &db_error( "prepare", $!, $select_statement );
  $sth->execute
    || &db_error( "execute", $!, $select_statement );

  while ( my $hash_ref = $sth->fetchrow_hashref('NAME_lc') ) {
    foreach $type (@types) {
      $hash_ref->{"$type"} = [ &parse_codes( "$hash_ref->{$type}" ) ];
    }
    push @lists, { %$hash_ref };
    ++$hit_count;
  }

  if ( $hit_count < 1 ) {
    my $vars = { session_info => $session_info,
		 editor => $editor,
		 descriptions => $descriptions,
		 filters => $filters,
		 searchtext => $form_info->{searchtext},
		 total_count => $total_count,
		 hit_count => $hit_count,
		 page => "search",
	       };

    # process the cookie and print the page
    &process_cookie( $session_info->{session_id} );
    &process_template('search_results.html', $vars );
    return;
  } elsif ( $hit_count > 40 ) {
    my $vars = { session_info => $session_info,
		 editor => $editor,
		 descriptions => $descriptions,
		 filters => $filters,
		 searchtext => $form_info->{searchtext},
		 total_count => $total_count,
		 hit_count => $hit_count,
		 page => "search",
	       };

    # process the cookie and print the page
    &process_cookie( $session_info->{session_id} );
    &process_template('search_results.html', $vars );
    return;
  } else {
    # copy data into the outgoing hash
    my $vars = { session_info => $session_info,
		 editor => $editor,
		 descriptions => $descriptions,
		 searchtext => $form_info->{searchtext},
		 filters => $filters,
		 total_count => $total_count,
		 hit_count => $hit_count,
		 lists => \@lists,
		 page => "search",
	       };

    # process the cookie and print the page
    &process_cookie( $session_info->{session_id} );
    &process_template('search_results.html', $vars );
    return;
  }
}

## error -- a little wrapper around the dg_error script
sub error
  { &dg_error( 'script', 'browse', @_ ); }


## Shared code within this file
#################################################
sub get_filters {
  my ( $type ) ;
  my %filters;

  foreach $type (@types) {

    if (defined $form_info->{"$type"}) {
      if ($type eq $form_info->{browseby})
      { &error( 'browsetype', 'invalidbrowseby' ); }
	$filters{"$type"} = [ $q->param("$type") ];
    }
  }
  return (\%filters);
}

# build the condition fragment for the database calls
sub build_conditions_fragment {
  my ( $filters ) = @_;
  my ( $type, $codes, $code );
  my $filters_size = scalar(keys( %$filters ));
  my $conditions_fragment;

  $i = 1;
  while ( ( $type, $codes ) = each( %$filters ) ) {
    $ii = 0;
    foreach $code ( @$codes ) {
      $conditions_fragment .= qq( $type LIKE "%$code%" );
      $conditions_fragment .= q( AND ) unless ( $ii == $#$codes );
      ++$ii;
    }
    $conditions_fragment .= q( AND ) unless ( $i == $filters_size );
    ++$i;
  }
  return( $conditions_fragment );
}
