package Calf::Auth;
# Auth.pm -- Mako Hill <mako@debian.org>
# written for Organizers' Collaboaritve -- 2001
# this code is licensed unde the GNU GPL, see COPYING for more details

use Calf::Config qw( :base );
use Calf::Utils qw( :base :template );
use Exporter;

@ISA = ('Exporter');

@EXPORT_OK = qw( &valid_id &generate_id &invalid_access
                 &verify_editorship &retrieve_editor_codes );

%EXPORT_TAGS = ( base => [ qw( &valid_id &generate_id &invalid_access
                               &verify_editorship &retrieve_editor_codes ) ] );

## check to see if an a user requesting a page is logged in and
## currently authenticated to recieved a protected page
##### args: the query and database handles
sub valid_id {
  my ($q, $dbh ) = @_;

  # attempt to grab a session id from a cookie or just say it's invalid

  my $unsafe_id = $q->cookie( 'session_id' )
    || return undef;

  # is it in a valid form?
  # if so, does it reflect a valid session_id in the db?
  # if so, return the valid info, ir not, claim it's invalid
  if ($unsafe_id =~ s/^(.+)$/$1/ ) {
    my $select_statement = qq/ SELECT username, fullname,
 			       email, superuser
			       FROM editors
			       WHERE session_id = "$unsafe_id" /;

    my $sth = $dbh->prepare( $select_statement )
      || &db_error( "prepare", $!, $select_statement );
    my $rv = $sth->execute
      || &db_error( "execute", $!, $select_statement );
    return ( undef ) if ( ( $rv < 1 ) || ( $rv > 1 ) );

    my ( $db_username, $db_fullname, $db_email, $db_superuser )
      = ( $sth->fetchrow_array )[0,1,2,3];

    my $session_info = { username => $db_username,
			 fullname => $db_fullname,
			 email => $db_email,
			 superuser => $db_superuser,
			 session_id => $unsafe_id };
    return $session_info;
  }
  else { return undef; }
}

## use apache's unique_id ENVVAR or create one and return it as a
## session id..
#
# this is not truly secure (it's predictable) but it will be unique
# and for our purposes that will probably be fine and can be easily
# replaced later if that's what needs to be done or if security seems
# to be of higher importance
sub generate_id {
  # use Apache's mod_unique_id
  return $ENV{UNIQUE_ID} if exists $ENV{UNIQUE_ID};

  require Digest::MD5;

  my $md5 = new Digest::MD5;
  my $remote = $ENV{REMOTE_ADDR} . $ENV{REMOTE_PORT};

  my $id = $md5->md5_base64( time, $$, $remote );
  $id =~ tr|+/=|-_.|; # make the md5 hash url friendly
  return $id;
}

# take the list_id, session_info, and the database handle and  return the
# that list falls into one of the editors issues or constits
sub verify_editorship ($$$) {
  my ( $list_id, $session_info, $dbh ) = @_;
  my ($type, $fieldname);

  return( 1 ) if ($session_info->{superuser} == 1);
  foreach $type (@types) {
    my $editor_codes = ( &retrieve_editor_codes( $session_info, $type, $dbh ));

    $fieldname = lc($type);
    my $raw_list_codes = ($dbh->selectrow_array(qq{ SELECT $fieldname FROM
						    calf_data
						    WHERE
						    id = $list_id
						  }))[0];
    foreach $editor_code ( (keys %$editor_codes) ) {
      if ( $raw_list_codes =~ m/$editor_code/ ) { return("1") } # SUCCESS!!
    }
  }

  # if we've gotten this far, we've failed all "types" and we should return a
  # big fat undef
  return();
}

sub invalid_access {
  &process_template('invalid_access.html', {} );
}

## grab the codes for a particular editor's issues or constits
##### arg: session_info, type, and database handle
sub retrieve_editor_codes ($$$) {
  my ( $session_info, $type, $dbh ) = @_;
  my ( %codes, $categories, $code ) = ();

  my $raw_codes = 
    ($dbh->selectrow_array(qq{ SELECT $type FROM editors WHERE
			       session_id = "$session_info->{session_id}"
			     }))[0];

  $categories = &make_type_hash( $type, $dbh );

  foreach $code (&parse_codes($raw_codes)) {
    $codes{$code} = $categories->{$code};
  }
  return( \%codes );
}

1;
