package Calf::Config;
# Config.pm -- Mako Hill <mako@debian.org>
# written for Organizers' Collaborative -- 2001
# this code is licensed unde the GNU GPL, see COPYING for more details

# this is the configuration module for the site and it's written in
# pure perl

use Exporter;
@ISA = ('Exporter');
@EXPORT_OK = qw( @types %type_defs $rec_thresh $rec_thresh_editor
                 $include_path $config_file
                 $editable_fields @required_fields
                 %dbconfig @all_fields
		 $webserver_calfroot );

%EXPORT_TAGS = (# basic information that everyone will need
		base => [ qw( @types %type_defs $rec_thresh @all_fields ) ],
		# data that will be necessary when generating dynamic html
		template_config => [ qw( $include_path $config_file $webserver_calfroot ) ],
		# items that are necessary when dealing with particular items
		item => [ qw( $editable_fields @required_fields ) ] );

## Configuration Options and Shared Variables
#############################################
# this hash holds the configuration options for the database
%dbconfig =
  ( # Edit the values below to configure the database
   driver   => "mysql",
   name     => "calf_demo",
   hostname => "localhost",
   username => "calf_demo",
   password => "changme"
  );

# the include path for the templating system
$include_path = '/var/www/calf.democracygroups.org/demonstration/src:/var/www/calf.democracygroups.org/demonstration/lib/templates';

# on most systems, / will be what you are looking for. the trailing /
# is necessary
$webserver_calfroot = '/demonstration/';

# these are the fields/tables in the database that should be treated
# as "types for categorization"
@types = qw( camp constit );

# these are full names that should be associated with the fields
# listed in @types above
%type_defs = ( camp => "camp",
               constit => "constituency" );

# fields that are required while submitting new lists
@required_fields = qw( name camp constit );

# recommended thresholds
$rec_thresh = 2;   # less than or equal to this number, lists
                   # will be displayed in the main site
$rec_thresh_editor = 2; # less than or equal to this number, lists will
                        # be displayed to editors

# this is a list of every field editable through the site and a host
# of options associated with each field
$editable_fields =
 [
  # these are necessary and used internally by CALF
  { name => "approved", title => "Approved List", type => "radio",
    size => 1, choices => { yes => 1, no => 0 }, editoronly => 1 },
  { name => "contactname", title => "Your Name", type=> "text", size => 40,
    required => 1, submitonly => 1 },
  { name => "contactemail", title => "Your Email", type=> "text", size => 40,
    required => 1, submitonly => 1 },
  { name => "keywords", title => "Keywords",
    type => "text", size => 50 },  
  { name => "name", title => "Organization Name", type => "text",
    size => 35, required => 1 },


  # these are fields you'll want to customize (remove/add/modify them)
  # according to how you're using your database to fit your needs
  { name => "description", title => "Description (255 chars max)",
    type => "text", size => 255, required => 1 },
  { name => "city", title => "City", type => "text", size => 25 },
  { name => "state", title => "State",
    type => "select", size => 2, selectoptions=> "options_state"},
  { name => "camp", title => "Free Software/Open Source Standing", 
    type => "categorization", dbtable => "camp", size => 14, required => 1 },
  { name => "constit", title => "Constituencies", type => "categorization",
    dbtable => "constit", size => 14, required => 1 },
  { name => "founder", title => "Founder", type => "text", size => 40 },
  { name => "contact", title => "Contact", type => "text", size => 40 },
  { name => "orgwebsite", title => "Web Site of Associated Group",
    type => "text", size => 70 },
 ];

@all_fields = qw( id recommended updated entered
                  editornotify lastchange editorcomments lasteditor );
foreach $field (@$editable_fields) 
  { push( @all_fields, $field->{name}); }

1;
